------------------------------------------------------------------------------
-- Project name : Common DSP
--
-- Product name : N/A
--
-- Entity name  : sine_pkg
--
-- File name    : sine_pkg.vhd
--
-- Language     : VHDL-93
--
-- Library      : 
--
-- Tools        : Entry     = Altera Quartus II v9.0 sp1
--                Synthesis = Altera Quartus II v9.0 sp1   
--                Simulator = Modelsim 6.5 PE

--
-- Company      : Skipper
-----------------------------------------------------------------------------------
-- Purpose      :  The LUT values are generated via a Doulos PERL script:
--                 
--                 https://www.doulos.com/knowhow/vhdl_designers_guide/models/sine_wave_generator/ 
--
--                 A simple function has been coded to convert those scalar integer values to 
--                 an array of unsigned                 
---------------------------------------------------------------------------------- 
   
-- Note          : 
--
-- Limitations   : 
--
-- Errors        : None
--
-- Dependencies  : ieee, Doulos 
--
-- ----------------------------------------------------------------------------
--  Revision list
--  Version Author Date      Changes
--          !      !         !
--  1.0     JCP    23-07-14  New version
-- ----------------------------------------------------------------------------
library ieee;
library std;

use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use std.textio.all;

package sine_pkg is

  -- Mem initialization related
  constant c_hmg_mem_size  : integer := 1000;  
  constant c_hmg_data_width: integer := 16;
  type mem_type_hamming is ARRAY (0 to c_hmg_mem_size-1) of unsigned(c_hmg_data_width - 1 downto 0);

  impure function init_mem_hamming(mif_file_name : in string) RETURN mem_type_hamming; 


  constant max_table_value : integer := 2**16 -1;
  subtype table_value_type is integer range 0 to max_table_value;

  constant c_max_table_index: integer := 2**10 - 1;
  subtype table_index_type is integer range 0 to c_max_table_index;
  
  type t_arr_10x16_uns is array (0 to c_max_table_index) of unsigned(16 downto 0); 

  function get_table_value(table_index: table_index_type) return table_value_type;
  
  function get_uns_array(max_table_index: table_index_type) return  t_arr_10x16_uns;
  
  
  constant c_phi_bit_1msps: integer := 12;
  constant c_adc_data_size: integer := 16;
  constant c_phase_bits_1msps: integer := 12;
  
end;

package body sine_pkg is
   
  impure function init_mem_hamming(mif_file_name : in string) RETURN mem_type_hamming is
    file mif_file : text OPEN read_mode is mif_file_name;
    variable mif_line : line;
    variable temp_bv : bit_vector(c_hmg_data_width-1 downto 0);
    variable temp_int  : integer; 
    variable temp_mem : mem_type_hamming;
  BEGIN
    for i in mem_type_hamming'range loop
        readline(mif_file, mif_line);
        read(mif_line, temp_int);
        temp_mem(i) := to_unsigned(temp_int,c_hmg_data_width);
    end loop;
    return temp_mem;
  end function;  

  function get_table_value (table_index: table_index_type) return table_value_type is
    variable table_value: table_value_type;
  begin
    case table_index is
      when 0 =>
        table_value := 50;
      when 1 =>
        table_value := 151;
      when 2 =>
        table_value := 251;
      when 3 =>
        table_value := 352;
      when 4 =>
        table_value := 452;
      when 5 =>
        table_value := 553;
      when 6 =>
        table_value := 653;
      when 7 =>
        table_value := 754;
      when 8 =>
        table_value := 854;
      when 9 =>
        table_value := 955;
      when 10 =>
        table_value := 1056;
      when 11 =>
        table_value := 1156;
      when 12 =>
        table_value := 1257;
      when 13 =>
        table_value := 1357;
      when 14 =>
        table_value := 1458;
      when 15 =>
        table_value := 1558;
      when 16 =>
        table_value := 1659;
      when 17 =>
        table_value := 1759;
      when 18 =>
        table_value := 1860;
      when 19 =>
        table_value := 1960;
      when 20 =>
        table_value := 2061;
      when 21 =>
        table_value := 2161;
      when 22 =>
        table_value := 2261;
      when 23 =>
        table_value := 2362;
      when 24 =>
        table_value := 2462;
      when 25 =>
        table_value := 2563;
      when 26 =>
        table_value := 2663;
      when 27 =>
        table_value := 2764;
      when 28 =>
        table_value := 2864;
      when 29 =>
        table_value := 2965;
      when 30 =>
        table_value := 3065;
      when 31 =>
        table_value := 3165;
      when 32 =>
        table_value := 3266;
      when 33 =>
        table_value := 3366;
      when 34 =>
        table_value := 3467;
      when 35 =>
        table_value := 3567;
      when 36 =>
        table_value := 3667;
      when 37 =>
        table_value := 3768;
      when 38 =>
        table_value := 3868;
      when 39 =>
        table_value := 3968;
      when 40 =>
        table_value := 4069;
      when 41 =>
        table_value := 4169;
      when 42 =>
        table_value := 4269;
      when 43 =>
        table_value := 4370;
      when 44 =>
        table_value := 4470;
      when 45 =>
        table_value := 4570;
      when 46 =>
        table_value := 4671;
      when 47 =>
        table_value := 4771;
      when 48 =>
        table_value := 4871;
      when 49 =>
        table_value := 4971;
      when 50 =>
        table_value := 5072;
      when 51 =>
        table_value := 5172;
      when 52 =>
        table_value := 5272;
      when 53 =>
        table_value := 5372;
      when 54 =>
        table_value := 5472;
      when 55 =>
        table_value := 5573;
      when 56 =>
        table_value := 5673;
      when 57 =>
        table_value := 5773;
      when 58 =>
        table_value := 5873;
      when 59 =>
        table_value := 5973;
      when 60 =>
        table_value := 6073;
      when 61 =>
        table_value := 6173;
      when 62 =>
        table_value := 6273;
      when 63 =>
        table_value := 6374;
      when 64 =>
        table_value := 6474;
      when 65 =>
        table_value := 6574;
      when 66 =>
        table_value := 6674;
      when 67 =>
        table_value := 6774;
      when 68 =>
        table_value := 6874;
      when 69 =>
        table_value := 6974;
      when 70 =>
        table_value := 7074;
      when 71 =>
        table_value := 7173;
      when 72 =>
        table_value := 7273;
      when 73 =>
        table_value := 7373;
      when 74 =>
        table_value := 7473;
      when 75 =>
        table_value := 7573;
      when 76 =>
        table_value := 7673;
      when 77 =>
        table_value := 7773;
      when 78 =>
        table_value := 7873;
      when 79 =>
        table_value := 7972;
      when 80 =>
        table_value := 8072;
      when 81 =>
        table_value := 8172;
      when 82 =>
        table_value := 8272;
      when 83 =>
        table_value := 8371;
      when 84 =>
        table_value := 8471;
      when 85 =>
        table_value := 8571;
      when 86 =>
        table_value := 8670;
      when 87 =>
        table_value := 8770;
      when 88 =>
        table_value := 8870;
      when 89 =>
        table_value := 8969;
      when 90 =>
        table_value := 9069;
      when 91 =>
        table_value := 9168;
      when 92 =>
        table_value := 9268;
      when 93 =>
        table_value := 9367;
      when 94 =>
        table_value := 9467;
      when 95 =>
        table_value := 9566;
      when 96 =>
        table_value := 9666;
      when 97 =>
        table_value := 9765;
      when 98 =>
        table_value := 9865;
      when 99 =>
        table_value := 9964;
      when 100 =>
        table_value := 10063;
      when 101 =>
        table_value := 10163;
      when 102 =>
        table_value := 10262;
      when 103 =>
        table_value := 10361;
      when 104 =>
        table_value := 10460;
      when 105 =>
        table_value := 10560;
      when 106 =>
        table_value := 10659;
      when 107 =>
        table_value := 10758;
      when 108 =>
        table_value := 10857;
      when 109 =>
        table_value := 10956;
      when 110 =>
        table_value := 11055;
      when 111 =>
        table_value := 11154;
      when 112 =>
        table_value := 11254;
      when 113 =>
        table_value := 11353;
      when 114 =>
        table_value := 11452;
      when 115 =>
        table_value := 11550;
      when 116 =>
        table_value := 11649;
      when 117 =>
        table_value := 11748;
      when 118 =>
        table_value := 11847;
      when 119 =>
        table_value := 11946;
      when 120 =>
        table_value := 12045;
      when 121 =>
        table_value := 12144;
      when 122 =>
        table_value := 12243;
      when 123 =>
        table_value := 12341;
      when 124 =>
        table_value := 12440;
      when 125 =>
        table_value := 12539;
      when 126 =>
        table_value := 12637;
      when 127 =>
        table_value := 12736;
      when 128 =>
        table_value := 12835;
      when 129 =>
        table_value := 12933;
      when 130 =>
        table_value := 13032;
      when 131 =>
        table_value := 13130;
      when 132 =>
        table_value := 13229;
      when 133 =>
        table_value := 13327;
      when 134 =>
        table_value := 13425;
      when 135 =>
        table_value := 13524;
      when 136 =>
        table_value := 13622;
      when 137 =>
        table_value := 13721;
      when 138 =>
        table_value := 13819;
      when 139 =>
        table_value := 13917;
      when 140 =>
        table_value := 14015;
      when 141 =>
        table_value := 14113;
      when 142 =>
        table_value := 14212;
      when 143 =>
        table_value := 14310;
      when 144 =>
        table_value := 14408;
      when 145 =>
        table_value := 14506;
      when 146 =>
        table_value := 14604;
      when 147 =>
        table_value := 14702;
      when 148 =>
        table_value := 14800;
      when 149 =>
        table_value := 14898;
      when 150 =>
        table_value := 14996;
      when 151 =>
        table_value := 15093;
      when 152 =>
        table_value := 15191;
      when 153 =>
        table_value := 15289;
      when 154 =>
        table_value := 15387;
      when 155 =>
        table_value := 15485;
      when 156 =>
        table_value := 15582;
      when 157 =>
        table_value := 15680;
      when 158 =>
        table_value := 15777;
      when 159 =>
        table_value := 15875;
      when 160 =>
        table_value := 15972;
      when 161 =>
        table_value := 16070;
      when 162 =>
        table_value := 16167;
      when 163 =>
        table_value := 16265;
      when 164 =>
        table_value := 16362;
      when 165 =>
        table_value := 16459;
      when 166 =>
        table_value := 16557;
      when 167 =>
        table_value := 16654;
      when 168 =>
        table_value := 16751;
      when 169 =>
        table_value := 16848;
      when 170 =>
        table_value := 16946;
      when 171 =>
        table_value := 17043;
      when 172 =>
        table_value := 17140;
      when 173 =>
        table_value := 17237;
      when 174 =>
        table_value := 17334;
      when 175 =>
        table_value := 17431;
      when 176 =>
        table_value := 17527;
      when 177 =>
        table_value := 17624;
      when 178 =>
        table_value := 17721;
      when 179 =>
        table_value := 17818;
      when 180 =>
        table_value := 17915;
      when 181 =>
        table_value := 18011;
      when 182 =>
        table_value := 18108;
      when 183 =>
        table_value := 18205;
      when 184 =>
        table_value := 18301;
      when 185 =>
        table_value := 18398;
      when 186 =>
        table_value := 18494;
      when 187 =>
        table_value := 18590;
      when 188 =>
        table_value := 18687;
      when 189 =>
        table_value := 18783;
      when 190 =>
        table_value := 18879;
      when 191 =>
        table_value := 18976;
      when 192 =>
        table_value := 19072;
      when 193 =>
        table_value := 19168;
      when 194 =>
        table_value := 19264;
      when 195 =>
        table_value := 19360;
      when 196 =>
        table_value := 19456;
      when 197 =>
        table_value := 19552;
      when 198 =>
        table_value := 19648;
      when 199 =>
        table_value := 19744;
      when 200 =>
        table_value := 19840;
      when 201 =>
        table_value := 19936;
      when 202 =>
        table_value := 20031;
      when 203 =>
        table_value := 20127;
      when 204 =>
        table_value := 20223;
      when 205 =>
        table_value := 20318;
      when 206 =>
        table_value := 20414;
      when 207 =>
        table_value := 20509;
      when 208 =>
        table_value := 20605;
      when 209 =>
        table_value := 20700;
      when 210 =>
        table_value := 20796;
      when 211 =>
        table_value := 20891;
      when 212 =>
        table_value := 20986;
      when 213 =>
        table_value := 21081;
      when 214 =>
        table_value := 21177;
      when 215 =>
        table_value := 21272;
      when 216 =>
        table_value := 21367;
      when 217 =>
        table_value := 21462;
      when 218 =>
        table_value := 21557;
      when 219 =>
        table_value := 21652;
      when 220 =>
        table_value := 21746;
      when 221 =>
        table_value := 21841;
      when 222 =>
        table_value := 21936;
      when 223 =>
        table_value := 22031;
      when 224 =>
        table_value := 22125;
      when 225 =>
        table_value := 22220;
      when 226 =>
        table_value := 22315;
      when 227 =>
        table_value := 22409;
      when 228 =>
        table_value := 22503;
      when 229 =>
        table_value := 22598;
      when 230 =>
        table_value := 22692;
      when 231 =>
        table_value := 22786;
      when 232 =>
        table_value := 22881;
      when 233 =>
        table_value := 22975;
      when 234 =>
        table_value := 23069;
      when 235 =>
        table_value := 23163;
      when 236 =>
        table_value := 23257;
      when 237 =>
        table_value := 23351;
      when 238 =>
        table_value := 23445;
      when 239 =>
        table_value := 23539;
      when 240 =>
        table_value := 23633;
      when 241 =>
        table_value := 23726;
      when 242 =>
        table_value := 23820;
      when 243 =>
        table_value := 23914;
      when 244 =>
        table_value := 24007;
      when 245 =>
        table_value := 24101;
      when 246 =>
        table_value := 24194;
      when 247 =>
        table_value := 24288;
      when 248 =>
        table_value := 24381;
      when 249 =>
        table_value := 24474;
      when 250 =>
        table_value := 24567;
      when 251 =>
        table_value := 24661;
      when 252 =>
        table_value := 24754;
      when 253 =>
        table_value := 24847;
      when 254 =>
        table_value := 24940;
      when 255 =>
        table_value := 25033;
      when 256 =>
        table_value := 25126;
      when 257 =>
        table_value := 25218;
      when 258 =>
        table_value := 25311;
      when 259 =>
        table_value := 25404;
      when 260 =>
        table_value := 25497;
      when 261 =>
        table_value := 25589;
      when 262 =>
        table_value := 25682;
      when 263 =>
        table_value := 25774;
      when 264 =>
        table_value := 25866;
      when 265 =>
        table_value := 25959;
      when 266 =>
        table_value := 26051;
      when 267 =>
        table_value := 26143;
      when 268 =>
        table_value := 26235;
      when 269 =>
        table_value := 26328;
      when 270 =>
        table_value := 26420;
      when 271 =>
        table_value := 26512;
      when 272 =>
        table_value := 26603;
      when 273 =>
        table_value := 26695;
      when 274 =>
        table_value := 26787;
      when 275 =>
        table_value := 26879;
      when 276 =>
        table_value := 26970;
      when 277 =>
        table_value := 27062;
      when 278 =>
        table_value := 27154;
      when 279 =>
        table_value := 27245;
      when 280 =>
        table_value := 27336;
      when 281 =>
        table_value := 27428;
      when 282 =>
        table_value := 27519;
      when 283 =>
        table_value := 27610;
      when 284 =>
        table_value := 27701;
      when 285 =>
        table_value := 27792;
      when 286 =>
        table_value := 27883;
      when 287 =>
        table_value := 27974;
      when 288 =>
        table_value := 28065;
      when 289 =>
        table_value := 28156;
      when 290 =>
        table_value := 28247;
      when 291 =>
        table_value := 28337;
      when 292 =>
        table_value := 28428;
      when 293 =>
        table_value := 28519;
      when 294 =>
        table_value := 28609;
      when 295 =>
        table_value := 28700;
      when 296 =>
        table_value := 28790;
      when 297 =>
        table_value := 28880;
      when 298 =>
        table_value := 28970;
      when 299 =>
        table_value := 29061;
      when 300 =>
        table_value := 29151;
      when 301 =>
        table_value := 29241;
      when 302 =>
        table_value := 29331;
      when 303 =>
        table_value := 29420;
      when 304 =>
        table_value := 29510;
      when 305 =>
        table_value := 29600;
      when 306 =>
        table_value := 29690;
      when 307 =>
        table_value := 29779;
      when 308 =>
        table_value := 29869;
      when 309 =>
        table_value := 29958;
      when 310 =>
        table_value := 30047;
      when 311 =>
        table_value := 30137;
      when 312 =>
        table_value := 30226;
      when 313 =>
        table_value := 30315;
      when 314 =>
        table_value := 30404;
      when 315 =>
        table_value := 30493;
      when 316 =>
        table_value := 30582;
      when 317 =>
        table_value := 30671;
      when 318 =>
        table_value := 30760;
      when 319 =>
        table_value := 30849;
      when 320 =>
        table_value := 30937;
      when 321 =>
        table_value := 31026;
      when 322 =>
        table_value := 31114;
      when 323 =>
        table_value := 31203;
      when 324 =>
        table_value := 31291;
      when 325 =>
        table_value := 31380;
      when 326 =>
        table_value := 31468;
      when 327 =>
        table_value := 31556;
      when 328 =>
        table_value := 31644;
      when 329 =>
        table_value := 31732;
      when 330 =>
        table_value := 31820;
      when 331 =>
        table_value := 31908;
      when 332 =>
        table_value := 31995;
      when 333 =>
        table_value := 32083;
      when 334 =>
        table_value := 32171;
      when 335 =>
        table_value := 32258;
      when 336 =>
        table_value := 32346;
      when 337 =>
        table_value := 32433;
      when 338 =>
        table_value := 32521;
      when 339 =>
        table_value := 32608;
      when 340 =>
        table_value := 32695;
      when 341 =>
        table_value := 32782;
      when 342 =>
        table_value := 32869;
      when 343 =>
        table_value := 32956;
      when 344 =>
        table_value := 33043;
      when 345 =>
        table_value := 33130;
      when 346 =>
        table_value := 33216;
      when 347 =>
        table_value := 33303;
      when 348 =>
        table_value := 33389;
      when 349 =>
        table_value := 33476;
      when 350 =>
        table_value := 33562;
      when 351 =>
        table_value := 33649;
      when 352 =>
        table_value := 33735;
      when 353 =>
        table_value := 33821;
      when 354 =>
        table_value := 33907;
      when 355 =>
        table_value := 33993;
      when 356 =>
        table_value := 34079;
      when 357 =>
        table_value := 34165;
      when 358 =>
        table_value := 34251;
      when 359 =>
        table_value := 34336;
      when 360 =>
        table_value := 34422;
      when 361 =>
        table_value := 34507;
      when 362 =>
        table_value := 34593;
      when 363 =>
        table_value := 34678;
      when 364 =>
        table_value := 34763;
      when 365 =>
        table_value := 34848;
      when 366 =>
        table_value := 34934;
      when 367 =>
        table_value := 35019;
      when 368 =>
        table_value := 35104;
      when 369 =>
        table_value := 35188;
      when 370 =>
        table_value := 35273;
      when 371 =>
        table_value := 35358;
      when 372 =>
        table_value := 35442;
      when 373 =>
        table_value := 35527;
      when 374 =>
        table_value := 35611;
      when 375 =>
        table_value := 35696;
      when 376 =>
        table_value := 35780;
      when 377 =>
        table_value := 35864;
      when 378 =>
        table_value := 35948;
      when 379 =>
        table_value := 36032;
      when 380 =>
        table_value := 36116;
      when 381 =>
        table_value := 36200;
      when 382 =>
        table_value := 36284;
      when 383 =>
        table_value := 36367;
      when 384 =>
        table_value := 36451;
      when 385 =>
        table_value := 36535;
      when 386 =>
        table_value := 36618;
      when 387 =>
        table_value := 36701;
      when 388 =>
        table_value := 36785;
      when 389 =>
        table_value := 36868;
      when 390 =>
        table_value := 36951;
      when 391 =>
        table_value := 37034;
      when 392 =>
        table_value := 37117;
      when 393 =>
        table_value := 37199;
      when 394 =>
        table_value := 37282;
      when 395 =>
        table_value := 37365;
      when 396 =>
        table_value := 37447;
      when 397 =>
        table_value := 37530;
      when 398 =>
        table_value := 37612;
      when 399 =>
        table_value := 37694;
      when 400 =>
        table_value := 37777;
      when 401 =>
        table_value := 37859;
      when 402 =>
        table_value := 37941;
      when 403 =>
        table_value := 38023;
      when 404 =>
        table_value := 38105;
      when 405 =>
        table_value := 38186;
      when 406 =>
        table_value := 38268;
      when 407 =>
        table_value := 38350;
      when 408 =>
        table_value := 38431;
      when 409 =>
        table_value := 38512;
      when 410 =>
        table_value := 38594;
      when 411 =>
        table_value := 38675;
      when 412 =>
        table_value := 38756;
      when 413 =>
        table_value := 38837;
      when 414 =>
        table_value := 38918;
      when 415 =>
        table_value := 38999;
      when 416 =>
        table_value := 39080;
      when 417 =>
        table_value := 39160;
      when 418 =>
        table_value := 39241;
      when 419 =>
        table_value := 39321;
      when 420 =>
        table_value := 39402;
      when 421 =>
        table_value := 39482;
      when 422 =>
        table_value := 39562;
      when 423 =>
        table_value := 39642;
      when 424 =>
        table_value := 39722;
      when 425 =>
        table_value := 39802;
      when 426 =>
        table_value := 39882;
      when 427 =>
        table_value := 39962;
      when 428 =>
        table_value := 40041;
      when 429 =>
        table_value := 40121;
      when 430 =>
        table_value := 40200;
      when 431 =>
        table_value := 40280;
      when 432 =>
        table_value := 40359;
      when 433 =>
        table_value := 40438;
      when 434 =>
        table_value := 40517;
      when 435 =>
        table_value := 40596;
      when 436 =>
        table_value := 40675;
      when 437 =>
        table_value := 40754;
      when 438 =>
        table_value := 40832;
      when 439 =>
        table_value := 40911;
      when 440 =>
        table_value := 40989;
      when 441 =>
        table_value := 41068;
      when 442 =>
        table_value := 41146;
      when 443 =>
        table_value := 41224;
      when 444 =>
        table_value := 41302;
      when 445 =>
        table_value := 41380;
      when 446 =>
        table_value := 41458;
      when 447 =>
        table_value := 41536;
      when 448 =>
        table_value := 41614;
      when 449 =>
        table_value := 41691;
      when 450 =>
        table_value := 41769;
      when 451 =>
        table_value := 41846;
      when 452 =>
        table_value := 41924;
      when 453 =>
        table_value := 42001;
      when 454 =>
        table_value := 42078;
      when 455 =>
        table_value := 42155;
      when 456 =>
        table_value := 42232;
      when 457 =>
        table_value := 42309;
      when 458 =>
        table_value := 42385;
      when 459 =>
        table_value := 42462;
      when 460 =>
        table_value := 42539;
      when 461 =>
        table_value := 42615;
      when 462 =>
        table_value := 42691;
      when 463 =>
        table_value := 42768;
      when 464 =>
        table_value := 42844;
      when 465 =>
        table_value := 42920;
      when 466 =>
        table_value := 42996;
      when 467 =>
        table_value := 43071;
      when 468 =>
        table_value := 43147;
      when 469 =>
        table_value := 43223;
      when 470 =>
        table_value := 43298;
      when 471 =>
        table_value := 43374;
      when 472 =>
        table_value := 43449;
      when 473 =>
        table_value := 43524;
      when 474 =>
        table_value := 43599;
      when 475 =>
        table_value := 43674;
      when 476 =>
        table_value := 43749;
      when 477 =>
        table_value := 43824;
      when 478 =>
        table_value := 43899;
      when 479 =>
        table_value := 43973;
      when 480 =>
        table_value := 44048;
      when 481 =>
        table_value := 44122;
      when 482 =>
        table_value := 44197;
      when 483 =>
        table_value := 44271;
      when 484 =>
        table_value := 44345;
      when 485 =>
        table_value := 44419;
      when 486 =>
        table_value := 44493;
      when 487 =>
        table_value := 44566;
      when 488 =>
        table_value := 44640;
      when 489 =>
        table_value := 44714;
      when 490 =>
        table_value := 44787;
      when 491 =>
        table_value := 44860;
      when 492 =>
        table_value := 44934;
      when 493 =>
        table_value := 45007;
      when 494 =>
        table_value := 45080;
      when 495 =>
        table_value := 45153;
      when 496 =>
        table_value := 45225;
      when 497 =>
        table_value := 45298;
      when 498 =>
        table_value := 45371;
      when 499 =>
        table_value := 45443;
      when 500 =>
        table_value := 45516;
      when 501 =>
        table_value := 45588;
      when 502 =>
        table_value := 45660;
      when 503 =>
        table_value := 45732;
      when 504 =>
        table_value := 45804;
      when 505 =>
        table_value := 45876;
      when 506 =>
        table_value := 45948;
      when 507 =>
        table_value := 46019;
      when 508 =>
        table_value := 46091;
      when 509 =>
        table_value := 46162;
      when 510 =>
        table_value := 46233;
      when 511 =>
        table_value := 46305;
      when 512 =>
        table_value := 46376;
      when 513 =>
        table_value := 46447;
      when 514 =>
        table_value := 46518;
      when 515 =>
        table_value := 46588;
      when 516 =>
        table_value := 46659;
      when 517 =>
        table_value := 46730;
      when 518 =>
        table_value := 46800;
      when 519 =>
        table_value := 46870;
      when 520 =>
        table_value := 46941;
      when 521 =>
        table_value := 47011;
      when 522 =>
        table_value := 47081;
      when 523 =>
        table_value := 47150;
      when 524 =>
        table_value := 47220;
      when 525 =>
        table_value := 47290;
      when 526 =>
        table_value := 47359;
      when 527 =>
        table_value := 47429;
      when 528 =>
        table_value := 47498;
      when 529 =>
        table_value := 47567;
      when 530 =>
        table_value := 47636;
      when 531 =>
        table_value := 47705;
      when 532 =>
        table_value := 47774;
      when 533 =>
        table_value := 47843;
      when 534 =>
        table_value := 47912;
      when 535 =>
        table_value := 47980;
      when 536 =>
        table_value := 48049;
      when 537 =>
        table_value := 48117;
      when 538 =>
        table_value := 48185;
      when 539 =>
        table_value := 48253;
      when 540 =>
        table_value := 48321;
      when 541 =>
        table_value := 48389;
      when 542 =>
        table_value := 48457;
      when 543 =>
        table_value := 48524;
      when 544 =>
        table_value := 48592;
      when 545 =>
        table_value := 48659;
      when 546 =>
        table_value := 48727;
      when 547 =>
        table_value := 48794;
      when 548 =>
        table_value := 48861;
      when 549 =>
        table_value := 48928;
      when 550 =>
        table_value := 48995;
      when 551 =>
        table_value := 49061;
      when 552 =>
        table_value := 49128;
      when 553 =>
        table_value := 49194;
      when 554 =>
        table_value := 49261;
      when 555 =>
        table_value := 49327;
      when 556 =>
        table_value := 49393;
      when 557 =>
        table_value := 49459;
      when 558 =>
        table_value := 49525;
      when 559 =>
        table_value := 49591;
      when 560 =>
        table_value := 49656;
      when 561 =>
        table_value := 49722;
      when 562 =>
        table_value := 49787;
      when 563 =>
        table_value := 49853;
      when 564 =>
        table_value := 49918;
      when 565 =>
        table_value := 49983;
      when 566 =>
        table_value := 50048;
      when 567 =>
        table_value := 50113;
      when 568 =>
        table_value := 50178;
      when 569 =>
        table_value := 50242;
      when 570 =>
        table_value := 50307;
      when 571 =>
        table_value := 50371;
      when 572 =>
        table_value := 50435;
      when 573 =>
        table_value := 50499;
      when 574 =>
        table_value := 50563;
      when 575 =>
        table_value := 50627;
      when 576 =>
        table_value := 50691;
      when 577 =>
        table_value := 50755;
      when 578 =>
        table_value := 50818;
      when 579 =>
        table_value := 50882;
      when 580 =>
        table_value := 50945;
      when 581 =>
        table_value := 51008;
      when 582 =>
        table_value := 51071;
      when 583 =>
        table_value := 51134;
      when 584 =>
        table_value := 51197;
      when 585 =>
        table_value := 51260;
      when 586 =>
        table_value := 51322;
      when 587 =>
        table_value := 51385;
      when 588 =>
        table_value := 51447;
      when 589 =>
        table_value := 51509;
      when 590 =>
        table_value := 51571;
      when 591 =>
        table_value := 51633;
      when 592 =>
        table_value := 51695;
      when 593 =>
        table_value := 51757;
      when 594 =>
        table_value := 51819;
      when 595 =>
        table_value := 51880;
      when 596 =>
        table_value := 51941;
      when 597 =>
        table_value := 52003;
      when 598 =>
        table_value := 52064;
      when 599 =>
        table_value := 52125;
      when 600 =>
        table_value := 52186;
      when 601 =>
        table_value := 52246;
      when 602 =>
        table_value := 52307;
      when 603 =>
        table_value := 52367;
      when 604 =>
        table_value := 52428;
      when 605 =>
        table_value := 52488;
      when 606 =>
        table_value := 52548;
      when 607 =>
        table_value := 52608;
      when 608 =>
        table_value := 52668;
      when 609 =>
        table_value := 52728;
      when 610 =>
        table_value := 52788;
      when 611 =>
        table_value := 52847;
      when 612 =>
        table_value := 52906;
      when 613 =>
        table_value := 52966;
      when 614 =>
        table_value := 53025;
      when 615 =>
        table_value := 53084;
      when 616 =>
        table_value := 53143;
      when 617 =>
        table_value := 53202;
      when 618 =>
        table_value := 53260;
      when 619 =>
        table_value := 53319;
      when 620 =>
        table_value := 53377;
      when 621 =>
        table_value := 53435;
      when 622 =>
        table_value := 53493;
      when 623 =>
        table_value := 53551;
      when 624 =>
        table_value := 53609;
      when 625 =>
        table_value := 53667;
      when 626 =>
        table_value := 53725;
      when 627 =>
        table_value := 53782;
      when 628 =>
        table_value := 53840;
      when 629 =>
        table_value := 53897;
      when 630 =>
        table_value := 53954;
      when 631 =>
        table_value := 54011;
      when 632 =>
        table_value := 54068;
      when 633 =>
        table_value := 54125;
      when 634 =>
        table_value := 54181;
      when 635 =>
        table_value := 54238;
      when 636 =>
        table_value := 54294;
      when 637 =>
        table_value := 54350;
      when 638 =>
        table_value := 54406;
      when 639 =>
        table_value := 54462;
      when 640 =>
        table_value := 54518;
      when 641 =>
        table_value := 54574;
      when 642 =>
        table_value := 54630;
      when 643 =>
        table_value := 54685;
      when 644 =>
        table_value := 54740;
      when 645 =>
        table_value := 54796;
      when 646 =>
        table_value := 54851;
      when 647 =>
        table_value := 54906;
      when 648 =>
        table_value := 54960;
      when 649 =>
        table_value := 55015;
      when 650 =>
        table_value := 55070;
      when 651 =>
        table_value := 55124;
      when 652 =>
        table_value := 55178;
      when 653 =>
        table_value := 55233;
      when 654 =>
        table_value := 55287;
      when 655 =>
        table_value := 55341;
      when 656 =>
        table_value := 55394;
      when 657 =>
        table_value := 55448;
      when 658 =>
        table_value := 55502;
      when 659 =>
        table_value := 55555;
      when 660 =>
        table_value := 55608;
      when 661 =>
        table_value := 55661;
      when 662 =>
        table_value := 55714;
      when 663 =>
        table_value := 55767;
      when 664 =>
        table_value := 55820;
      when 665 =>
        table_value := 55873;
      when 666 =>
        table_value := 55925;
      when 667 =>
        table_value := 55977;
      when 668 =>
        table_value := 56030;
      when 669 =>
        table_value := 56082;
      when 670 =>
        table_value := 56134;
      when 671 =>
        table_value := 56185;
      when 672 =>
        table_value := 56237;
      when 673 =>
        table_value := 56289;
      when 674 =>
        table_value := 56340;
      when 675 =>
        table_value := 56391;
      when 676 =>
        table_value := 56442;
      when 677 =>
        table_value := 56493;
      when 678 =>
        table_value := 56544;
      when 679 =>
        table_value := 56595;
      when 680 =>
        table_value := 56646;
      when 681 =>
        table_value := 56696;
      when 682 =>
        table_value := 56747;
      when 683 =>
        table_value := 56797;
      when 684 =>
        table_value := 56847;
      when 685 =>
        table_value := 56897;
      when 686 =>
        table_value := 56947;
      when 687 =>
        table_value := 56996;
      when 688 =>
        table_value := 57046;
      when 689 =>
        table_value := 57095;
      when 690 =>
        table_value := 57145;
      when 691 =>
        table_value := 57194;
      when 692 =>
        table_value := 57243;
      when 693 =>
        table_value := 57292;
      when 694 =>
        table_value := 57340;
      when 695 =>
        table_value := 57389;
      when 696 =>
        table_value := 57437;
      when 697 =>
        table_value := 57486;
      when 698 =>
        table_value := 57534;
      when 699 =>
        table_value := 57582;
      when 700 =>
        table_value := 57630;
      when 701 =>
        table_value := 57678;
      when 702 =>
        table_value := 57725;
      when 703 =>
        table_value := 57773;
      when 704 =>
        table_value := 57820;
      when 705 =>
        table_value := 57868;
      when 706 =>
        table_value := 57915;
      when 707 =>
        table_value := 57962;
      when 708 =>
        table_value := 58009;
      when 709 =>
        table_value := 58055;
      when 710 =>
        table_value := 58102;
      when 711 =>
        table_value := 58148;
      when 712 =>
        table_value := 58195;
      when 713 =>
        table_value := 58241;
      when 714 =>
        table_value := 58287;
      when 715 =>
        table_value := 58333;
      when 716 =>
        table_value := 58378;
      when 717 =>
        table_value := 58424;
      when 718 =>
        table_value := 58470;
      when 719 =>
        table_value := 58515;
      when 720 =>
        table_value := 58560;
      when 721 =>
        table_value := 58605;
      when 722 =>
        table_value := 58650;
      when 723 =>
        table_value := 58695;
      when 724 =>
        table_value := 58739;
      when 725 =>
        table_value := 58784;
      when 726 =>
        table_value := 58828;
      when 727 =>
        table_value := 58873;
      when 728 =>
        table_value := 58917;
      when 729 =>
        table_value := 58961;
      when 730 =>
        table_value := 59004;
      when 731 =>
        table_value := 59048;
      when 732 =>
        table_value := 59092;
      when 733 =>
        table_value := 59135;
      when 734 =>
        table_value := 59178;
      when 735 =>
        table_value := 59221;
      when 736 =>
        table_value := 59264;
      when 737 =>
        table_value := 59307;
      when 738 =>
        table_value := 59350;
      when 739 =>
        table_value := 59393;
      when 740 =>
        table_value := 59435;
      when 741 =>
        table_value := 59477;
      when 742 =>
        table_value := 59519;
      when 743 =>
        table_value := 59561;
      when 744 =>
        table_value := 59603;
      when 745 =>
        table_value := 59645;
      when 746 =>
        table_value := 59687;
      when 747 =>
        table_value := 59728;
      when 748 =>
        table_value := 59769;
      when 749 =>
        table_value := 59810;
      when 750 =>
        table_value := 59851;
      when 751 =>
        table_value := 59892;
      when 752 =>
        table_value := 59933;
      when 753 =>
        table_value := 59974;
      when 754 =>
        table_value := 60014;
      when 755 =>
        table_value := 60054;
      when 756 =>
        table_value := 60095;
      when 757 =>
        table_value := 60135;
      when 758 =>
        table_value := 60175;
      when 759 =>
        table_value := 60214;
      when 760 =>
        table_value := 60254;
      when 761 =>
        table_value := 60293;
      when 762 =>
        table_value := 60333;
      when 763 =>
        table_value := 60372;
      when 764 =>
        table_value := 60411;
      when 765 =>
        table_value := 60450;
      when 766 =>
        table_value := 60489;
      when 767 =>
        table_value := 60527;
      when 768 =>
        table_value := 60566;
      when 769 =>
        table_value := 60604;
      when 770 =>
        table_value := 60642;
      when 771 =>
        table_value := 60680;
      when 772 =>
        table_value := 60718;
      when 773 =>
        table_value := 60756;
      when 774 =>
        table_value := 60793;
      when 775 =>
        table_value := 60831;
      when 776 =>
        table_value := 60868;
      when 777 =>
        table_value := 60905;
      when 778 =>
        table_value := 60943;
      when 779 =>
        table_value := 60979;
      when 780 =>
        table_value := 61016;
      when 781 =>
        table_value := 61053;
      when 782 =>
        table_value := 61089;
      when 783 =>
        table_value := 61126;
      when 784 =>
        table_value := 61162;
      when 785 =>
        table_value := 61198;
      when 786 =>
        table_value := 61234;
      when 787 =>
        table_value := 61269;
      when 788 =>
        table_value := 61305;
      when 789 =>
        table_value := 61340;
      when 790 =>
        table_value := 61376;
      when 791 =>
        table_value := 61411;
      when 792 =>
        table_value := 61446;
      when 793 =>
        table_value := 61481;
      when 794 =>
        table_value := 61516;
      when 795 =>
        table_value := 61550;
      when 796 =>
        table_value := 61585;
      when 797 =>
        table_value := 61619;
      when 798 =>
        table_value := 61653;
      when 799 =>
        table_value := 61687;
      when 800 =>
        table_value := 61721;
      when 801 =>
        table_value := 61755;
      when 802 =>
        table_value := 61788;
      when 803 =>
        table_value := 61822;
      when 804 =>
        table_value := 61855;
      when 805 =>
        table_value := 61888;
      when 806 =>
        table_value := 61921;
      when 807 =>
        table_value := 61954;
      when 808 =>
        table_value := 61987;
      when 809 =>
        table_value := 62019;
      when 810 =>
        table_value := 62052;
      when 811 =>
        table_value := 62084;
      when 812 =>
        table_value := 62116;
      when 813 =>
        table_value := 62148;
      when 814 =>
        table_value := 62180;
      when 815 =>
        table_value := 62212;
      when 816 =>
        table_value := 62243;
      when 817 =>
        table_value := 62274;
      when 818 =>
        table_value := 62306;
      when 819 =>
        table_value := 62337;
      when 820 =>
        table_value := 62368;
      when 821 =>
        table_value := 62399;
      when 822 =>
        table_value := 62429;
      when 823 =>
        table_value := 62460;
      when 824 =>
        table_value := 62490;
      when 825 =>
        table_value := 62520;
      when 826 =>
        table_value := 62550;
      when 827 =>
        table_value := 62580;
      when 828 =>
        table_value := 62610;
      when 829 =>
        table_value := 62640;
      when 830 =>
        table_value := 62669;
      when 831 =>
        table_value := 62698;
      when 832 =>
        table_value := 62728;
      when 833 =>
        table_value := 62757;
      when 834 =>
        table_value := 62786;
      when 835 =>
        table_value := 62814;
      when 836 =>
        table_value := 62843;
      when 837 =>
        table_value := 62871;
      when 838 =>
        table_value := 62900;
      when 839 =>
        table_value := 62928;
      when 840 =>
        table_value := 62956;
      when 841 =>
        table_value := 62984;
      when 842 =>
        table_value := 63011;
      when 843 =>
        table_value := 63039;
      when 844 =>
        table_value := 63066;
      when 845 =>
        table_value := 63094;
      when 846 =>
        table_value := 63121;
      when 847 =>
        table_value := 63148;
      when 848 =>
        table_value := 63174;
      when 849 =>
        table_value := 63201;
      when 850 =>
        table_value := 63228;
      when 851 =>
        table_value := 63254;
      when 852 =>
        table_value := 63280;
      when 853 =>
        table_value := 63306;
      when 854 =>
        table_value := 63332;
      when 855 =>
        table_value := 63358;
      when 856 =>
        table_value := 63384;
      when 857 =>
        table_value := 63409;
      when 858 =>
        table_value := 63434;
      when 859 =>
        table_value := 63460;
      when 860 =>
        table_value := 63485;
      when 861 =>
        table_value := 63509;
      when 862 =>
        table_value := 63534;
      when 863 =>
        table_value := 63559;
      when 864 =>
        table_value := 63583;
      when 865 =>
        table_value := 63607;
      when 866 =>
        table_value := 63632;
      when 867 =>
        table_value := 63656;
      when 868 =>
        table_value := 63679;
      when 869 =>
        table_value := 63703;
      when 870 =>
        table_value := 63727;
      when 871 =>
        table_value := 63750;
      when 872 =>
        table_value := 63773;
      when 873 =>
        table_value := 63796;
      when 874 =>
        table_value := 63819;
      when 875 =>
        table_value := 63842;
      when 876 =>
        table_value := 63865;
      when 877 =>
        table_value := 63887;
      when 878 =>
        table_value := 63909;
      when 879 =>
        table_value := 63932;
      when 880 =>
        table_value := 63954;
      when 881 =>
        table_value := 63976;
      when 882 =>
        table_value := 63997;
      when 883 =>
        table_value := 64019;
      when 884 =>
        table_value := 64040;
      when 885 =>
        table_value := 64062;
      when 886 =>
        table_value := 64083;
      when 887 =>
        table_value := 64104;
      when 888 =>
        table_value := 64124;
      when 889 =>
        table_value := 64145;
      when 890 =>
        table_value := 64166;
      when 891 =>
        table_value := 64186;
      when 892 =>
        table_value := 64206;
      when 893 =>
        table_value := 64226;
      when 894 =>
        table_value := 64246;
      when 895 =>
        table_value := 64266;
      when 896 =>
        table_value := 64286;
      when 897 =>
        table_value := 64305;
      when 898 =>
        table_value := 64324;
      when 899 =>
        table_value := 64343;
      when 900 =>
        table_value := 64362;
      when 901 =>
        table_value := 64381;
      when 902 =>
        table_value := 64400;
      when 903 =>
        table_value := 64419;
      when 904 =>
        table_value := 64437;
      when 905 =>
        table_value := 64455;
      when 906 =>
        table_value := 64473;
      when 907 =>
        table_value := 64491;
      when 908 =>
        table_value := 64509;
      when 909 =>
        table_value := 64527;
      when 910 =>
        table_value := 64544;
      when 911 =>
        table_value := 64562;
      when 912 =>
        table_value := 64579;
      when 913 =>
        table_value := 64596;
      when 914 =>
        table_value := 64613;
      when 915 =>
        table_value := 64629;
      when 916 =>
        table_value := 64646;
      when 917 =>
        table_value := 64662;
      when 918 =>
        table_value := 64679;
      when 919 =>
        table_value := 64695;
      when 920 =>
        table_value := 64711;
      when 921 =>
        table_value := 64727;
      when 922 =>
        table_value := 64742;
      when 923 =>
        table_value := 64758;
      when 924 =>
        table_value := 64773;
      when 925 =>
        table_value := 64788;
      when 926 =>
        table_value := 64803;
      when 927 =>
        table_value := 64818;
      when 928 =>
        table_value := 64833;
      when 929 =>
        table_value := 64848;
      when 930 =>
        table_value := 64862;
      when 931 =>
        table_value := 64876;
      when 932 =>
        table_value := 64891;
      when 933 =>
        table_value := 64905;
      when 934 =>
        table_value := 64918;
      when 935 =>
        table_value := 64932;
      when 936 =>
        table_value := 64946;
      when 937 =>
        table_value := 64959;
      when 938 =>
        table_value := 64972;
      when 939 =>
        table_value := 64985;
      when 940 =>
        table_value := 64998;
      when 941 =>
        table_value := 65011;
      when 942 =>
        table_value := 65024;
      when 943 =>
        table_value := 65036;
      when 944 =>
        table_value := 65048;
      when 945 =>
        table_value := 65060;
      when 946 =>
        table_value := 65072;
      when 947 =>
        table_value := 65084;
      when 948 =>
        table_value := 65096;
      when 949 =>
        table_value := 65108;
      when 950 =>
        table_value := 65119;
      when 951 =>
        table_value := 65130;
      when 952 =>
        table_value := 65141;
      when 953 =>
        table_value := 65152;
      when 954 =>
        table_value := 65163;
      when 955 =>
        table_value := 65174;
      when 956 =>
        table_value := 65184;
      when 957 =>
        table_value := 65194;
      when 958 =>
        table_value := 65204;
      when 959 =>
        table_value := 65214;
      when 960 =>
        table_value := 65224;
      when 961 =>
        table_value := 65234;
      when 962 =>
        table_value := 65244;
      when 963 =>
        table_value := 65253;
      when 964 =>
        table_value := 65262;
      when 965 =>
        table_value := 65271;
      when 966 =>
        table_value := 65280;
      when 967 =>
        table_value := 65289;
      when 968 =>
        table_value := 65298;
      when 969 =>
        table_value := 65306;
      when 970 =>
        table_value := 65314;
      when 971 =>
        table_value := 65323;
      when 972 =>
        table_value := 65331;
      when 973 =>
        table_value := 65338;
      when 974 =>
        table_value := 65346;
      when 975 =>
        table_value := 65354;
      when 976 =>
        table_value := 65361;
      when 977 =>
        table_value := 65368;
      when 978 =>
        table_value := 65375;
      when 979 =>
        table_value := 65382;
      when 980 =>
        table_value := 65389;
      when 981 =>
        table_value := 65396;
      when 982 =>
        table_value := 65402;
      when 983 =>
        table_value := 65409;
      when 984 =>
        table_value := 65415;
      when 985 =>
        table_value := 65421;
      when 986 =>
        table_value := 65427;
      when 987 =>
        table_value := 65432;
      when 988 =>
        table_value := 65438;
      when 989 =>
        table_value := 65443;
      when 990 =>
        table_value := 65448;
      when 991 =>
        table_value := 65454;
      when 992 =>
        table_value := 65459;
      when 993 =>
        table_value := 65463;
      when 994 =>
        table_value := 65468;
      when 995 =>
        table_value := 65472;
      when 996 =>
        table_value := 65477;
      when 997 =>
        table_value := 65481;
      when 998 =>
        table_value := 65485;
      when 999 =>
        table_value := 65489;
      when 1000 =>
        table_value := 65492;
      when 1001 =>
        table_value := 65496;
      when 1002 =>
        table_value := 65499;
      when 1003 =>
        table_value := 65503;
      when 1004 =>
        table_value := 65506;
      when 1005 =>
        table_value := 65509;
      when 1006 =>
        table_value := 65511;
      when 1007 =>
        table_value := 65514;
      when 1008 =>
        table_value := 65516;
      when 1009 =>
        table_value := 65519;
      when 1010 =>
        table_value := 65521;
      when 1011 =>
        table_value := 65523;
      when 1012 =>
        table_value := 65525;
      when 1013 =>
        table_value := 65526;
      when 1014 =>
        table_value := 65528;
      when 1015 =>
        table_value := 65529;
      when 1016 =>
        table_value := 65531;
      when 1017 =>
        table_value := 65532;
      when 1018 =>
        table_value := 65533;
      when 1019 =>
        table_value := 65533;
      when 1020 =>
        table_value := 65534;
      when 1021 =>
        table_value := 65535;
      when 1022 =>
        table_value := 65535;
      when 1023 =>
        table_value := 65535;
    end case;
    return table_value;
  end;
  
    
  function get_uns_array(max_table_index: table_index_type) return  t_arr_10x16_uns is
  
  variable v_uns_array : t_arr_10x16_uns ;

  begin
    for i in 0 to max_table_index loop
      v_uns_array(i) := to_unsigned(get_table_value(i),17) ;      
    end loop;
    
    return v_uns_array;
  end;   
   
end;